@@ -11,7 +11,7 @@ use tokio::io::{AsyncRead, AsyncWrite};
 /// broadcast messages.
 pub async fn connect<T>(transport: T) -> Result<Context, Error>
 where
-    T: AsyncRead + AsyncWrite + Unpin + Send + 'static,
+    T: AsyncRead + AsyncWrite + Debug + Unpin + Send + 'static,
 {
     connect_slave(transport, Slave::broadcast()).await
 }
@@ -19,7 +19,7 @@ where
 /// Connect to any kind of Modbus slave device.
 pub async fn connect_slave<T>(transport: T, slave: Slave) -> Result<Context, Error>
 where
-    T: AsyncRead + AsyncWrite + Unpin + Send + 'static,
+    T: AsyncRead + AsyncWrite + Debug + Unpin + Send + 'static,
 {
     let client = service::rtu::connect_slave(transport, slave).await?;
 
