@@ -3,22 +3,20 @@ use super::{Context, Result};
 use crate::client::rtu::connect_slave as async_connect_slave;
 use crate::slave::Slave;
 
-use tokio_serial::{Serial, SerialPortSettings};
+use tokio_serial::{SerialPortBuilder, SerialStream};
 
 /// Connect to no particular Modbus slave device for sending
 /// broadcast messages.
-pub fn connect(tty_path: &str, settings: &SerialPortSettings) -> Result<Context> {
-    connect_slave(tty_path, settings, Slave::broadcast())
+pub fn connect(builder: &SerialPortBuilder) -> Result<Context> {
+    connect_slave(builder, Slave::broadcast())
 }
 
 /// Connect to any kind of Modbus slave device.
-pub fn connect_slave(
-    tty_path: &str,
-    settings: &SerialPortSettings,
-    slave: Slave,
-) -> Result<Context> {
-    let mut rt = tokio::runtime::Runtime::new()?;
-    let serial = Serial::from_path(tty_path, settings)?;
+pub fn connect_slave(builder: &SerialPortBuilder, slave: Slave) -> Result<Context> {
+    let rt = tokio::runtime::Builder::new_current_thread()
+        .enable_io()
+        .build()?;
+    let serial = SerialStream::open(builder).unwrap();
     let async_ctx = rt.block_on(async_connect_slave(serial, slave))?;
     let sync_ctx = Context {
         core: rt,
