@@ -1,3 +1,5 @@
+#[cfg(feature = "rtu")]
+pub mod rtu;
 #[cfg(feature = "tcp-server-unstable")]
 pub mod tcp;
 
